<?php class file extends baseobject {

	protected static $fields=array(
			'name'=>validator::ANY,
			'filename'=>validator::ANY,
			'mimetype'=>validator::ANY,
			'description'=>validator::ANY,
			'bytes'=>validator::INTEGER,
			'parentid'=>array(validator::REQUIRED, validator::INTEGER),
	);
	
	protected static $helpTexts=array(
	);

    /**
     * @param int $id
     * @return array|void
     * @throws BadRequestException
     * @throws NotFoundException
     * @throws ServerException
     */
	public static function getById($id){
		$file=parent::getById($id);
		$path=static::getPathToFile($file);
		if(!$file || !file_exists($path)){
			throw new NotFoundException('The file does not exist, or you do not have permission to see it');
		}
		header('Content-Type: '.$file['mimetype']);
		$handle=@fopen($path, 'r');
		@fpassthru($handle);
		@fclose($handle);
		exit;
	}

    /**
     * Required for interface compatibility but no implementation. Throws a BadRequestException.
     * @param $name
     * @return array|void
     * @throws BadRequestException
     */
	public static function getByName($name){
		throw new BadRequestException('getByName not implemented in file class');		
	}

    /**
     * @param array $request
     * @return array
     * @throws BadRequestException
     * @throws ForbiddenException
     * @throws NotFoundException
     * @throws ServerException
     */
	public static function create($request=array()){
	    if(empty($_FILES) || empty($_FILES['file'])){
	        throw new BadRequestException('file::create called but no file was supplied');
	    }
	    $file=$_FILES['file'];
	    $request['name']='file'.microtime();
	    $request['filename']=$file['name'];
	    $request['mimetype']=$file['type'];
	    $request['bytes']=$file['size'];
	    $parentObject=baseobject::getById($request['parentid']);
	    $request['projectid']=$parentObject['projectid'];
	    if(!in_array($parentObject['projectid'], session::getUpdateProjects())){
            $parentType=$parentObject['objecttype'];
            $canUpdate=forward_static_call_array(array($parentType, 'canUpdate'), array($request['parentid']));
            if(!$canUpdate){
                throw new ForbiddenException('You need "Update" permission on this project to attach files');
            }
	    }
	    $record=parent::create($request);
	    $record=$record['created'];
	    $fileDir=static::getFileParentDirectory($record);
	    if(!file_exists($fileDir)){
	        if(!@mkdir($fileDir, 0700, true)){
	            throw new ServerException('Could not save uploaded file into file storage area - could not make project directory');
	        }
	    }
	    $newPath=static::getPathToFile($record);
	    if(!@move_uploaded_file($file['tmp_name'], $newPath)){
	        throw new ServerException('Could not save uploaded file into file storage area');
	    }
	    
	    $params=array('name'=>'file'.$record['id'], 'id'=>$record['id']);
	    database::query('UPDATE file SET name=:name WHERE id=:id', $params);
	    
	    return array("created"=>parent::getById($record['id']) );
	}

    /**
     * @param array $request
     * @return array
     * @throws BadRequestException
     * @throws ForbiddenException
     * @throws NotFoundException
     * @throws ServerException
     */
	public static function createFromTempFile($request=array()){
	    $request['name']='file'.microtime();
	    if(!$request['tempfile']){
	        throw new BadRequestException('file::createFromTempFile called but no file was supplied');
	    }
        $request['bytes']=@filesize($request['tempfile']);
	    $parentObject=baseobject::getById($request['parentid']);
	    $request['projectid']=$parentObject['projectid'];
	    if(!in_array($parentObject['projectid'], session::getUpdateProjects())){
            $parentType=$parentObject['objecttype'];
            $canUpdate=forward_static_call_array(array($parentType, 'canUpdate'), array($request['parentid']));
            if(!$canUpdate){
                throw new ForbiddenException('You need "Update" permission on this project to attach files');
            }
	    }
	    $record=parent::create($request);
	    $record=$record['created'];
	    $fileDir=static::getFileParentDirectory($record);
	    if(!file_exists($fileDir)){
	        if(!@mkdir($fileDir, 0700, true)){
	            throw new ServerException('Could not save uploaded file into file storage area - could not make project directory');
	        }
	    }
	    $newPath=static::getPathToFile($record);
	    if(!@copy($request['tempfile'],$newPath)){
	        throw new ServerException('Could not save uploaded file into file storage area');
	    }
        @unlink($request['tempfile']);	    
	    $params=array('name'=>'file'.$record['id'], 'size'=>@filesize($newPath), 'id'=>$record['id']);
	    database::query('UPDATE file SET name=:name, bytes=:size WHERE id=:id', $params);
	    
	    return array("created"=>parent::getById($record['id']) );
	}
	
	private static function getFileParentDirectory($fileRecord){
		return rtrim(config::get('core_filestore'),'/').'/files/project'.$fileRecord['projectid'];
	}
	public static function getPathToFile($fileRecord){
		return static::getFileParentDirectory($fileRecord).'/file'.$fileRecord['id'];
	}
	
}